/*
 * @(#)UIHException.java
 *
 * Christopher Deckers (chrriis@brainlex.com)
 * http://chrriis.brainlex.com
 *
 * See the file "LICENSE.txt" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package chrriis.uihierarchy;

import java.io.PrintStream;

import chrriis.uihierarchy.compatibility.Compatibility;

/**
 * The super class of the UIHierarchy exceptions that can occur.
 * @version 1.0 2004.02.11
 * @author Christopher Deckers (chrriis@brainlex.com)
 */
public class UIHException extends RuntimeException {

  /** The accessor of the component. */
  protected SingleLevelAccessor accessor;

  /** The nested throwable. */
  protected Throwable target;

  /**
   * Construct a UIHException.
   * @param accessor The accessor of the component for which the exception was
   * raised.
   * @param message The message of the exception.
   */
  public UIHException(SingleLevelAccessor accessor, String message) {
    this(accessor, message, null);
  }

  /**
   * Construct a UIHException, nesting a cause.
   * @param accessor The accessor of the component for which the exception was
   * raised.
   * @param target The throwable that triggered this exception.
   */
  public UIHException(SingleLevelAccessor accessor, Throwable target) {
    this(accessor, null, target);
  }

  /**
   * Construct a UIHException, nesting a cause.
   * @param accessor The accessor of the component for which the exception was
   * raised.
   * @param message The message of the exception.
   * @param target The throwable that triggered this exception.
   */
  public UIHException(SingleLevelAccessor accessor, String message, Throwable target) {
    super("At level " + accessor.getDottedDecimalPath() + (message == null? "": ": " + message));
    this.accessor = accessor;
    this.target = target;
  }

  /**
   * Get an accessor to the level that failed.
   * @return The accessor.
   */
  public SingleLevelAccessor getAccessor() {
    return accessor;
  }

  /**
   * Get the throwable that triggered this constraints exception.
   * @return The cause.
   */
  public Throwable getCause() {
    return target;
  }

  /**
   * Prints the stack trace.
   */
  public void printStackTrace() {
    printStackTrace(System.err);
  }

  /**
   * Prints the stack trace to a print writer.
   * @param s The print writer.
   */
  public void printStackTrace(java.io.PrintWriter s) {
    synchronized(s) {
      super.printStackTrace(s);
      if(target != null && Compatibility.getJavaVersion() < Compatibility.JAVA_1_4) {
        s.print("Caused by: ");
        target.printStackTrace(s);
      }
    }
  }

  /**
   * Prints the stack trace to a print stream.
   * @param ps The print stream.
   */
  public void printStackTrace(PrintStream ps) {
    synchronized(ps) {
      super.printStackTrace(ps);
      if(target != null && Compatibility.getJavaVersion() < Compatibility.JAVA_1_4) {
        ps.print("Caused by: ");
        target.printStackTrace(ps);
      }
    }
  }

}
