/*
 * @(#)ArrayList.java
 * 
 * Christopher Deckers (chrriis@brainlex.com)
 * http://chrriis.brainlex.com
 * 
 * See the file "LICENSE.txt" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package chrriis.uihierarchy.compatibility;

/**
 * The arrayList compatible version.
 * @version 1.0 2004.01.11
 * @author Christopher Deckers (chrriis@brainlex.com)
 */
public class ArrayList implements List {

  /** The array list. */
  protected Object arrayList;

  /**
   * Constructs an empty list with an initial capacity of ten.
   */
  public ArrayList() {
    if(Compatibility.isJava1_1()) {
      arrayList = new ArrayList_1_1();
    } else {
      arrayList = new java.util.ArrayList();
    }
  }

  /**
   * Constructs a list containing the elements of the specified collection, in
   * the order they are returned by the collection's iterator.
   * @param c The collection.
   */
  public ArrayList(Collection c) {
    if(Compatibility.isJava1_1()) {
      arrayList = new ArrayList_1_1(c);
    } else {
      arrayList = new java.util.ArrayList(c.size());
      addAll(c);
    }
  }

  /**
   * Constructs an empty list with the specified initial capacity.
   * @param initialCapacity The initial capacity.
   */
  public ArrayList(int initialCapacity) {
    if(Compatibility.isJava1_1()) {
      arrayList = new ArrayList_1_1(initialCapacity);
    } else {
      arrayList = new java.util.ArrayList(initialCapacity);
    }
  }

  /**
   * Returns true if this collection contains the specified element.
   * @param o The object to look for.
   * @return True if found.
   */
  public boolean contains(Object o) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).contains(o);
    }
    return ((java.util.ArrayList)arrayList).contains(o);
  }

  /**
   * Returns the number of elements in this list.
   * @return The size.
   */
  public int size() {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).size();
    }
    return ((java.util.ArrayList)arrayList).size();
  }

  /**
   * Returns the element at the specified position in this list.
   * @param index The index.
   * @return The element.
   */
  public Object get(int index) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).get(index);
    }
    return ((java.util.ArrayList)arrayList).get(index);
  }

  /**
   * Returns an array containing all of the elements in this collection. 
   * @return The array.
   */
  public Object[] toArray() {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).toArray();
    }
    return ((java.util.ArrayList)arrayList).toArray();
  }

  /**
   * Returns an array containing all of the elements in this collection; the
   * runtime type of the returned array is that of the specified array.
   * @param a The array from which to get the type.
   * @return The array.
   */
  public Object[] toArray(Object[] a) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).toArray(a);
    }
    return ((java.util.ArrayList)arrayList).toArray(a);
  }

  /**
   * Appends the specified element to the end of this list (optional operation).
   * @param o The object to add.
   * @return True.
   */
  public boolean add(Object o) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).add(o);
    }
    return ((java.util.ArrayList)arrayList).add(o);
  }

  /**
   * Inserts the specified element at the specified position in this list
   * (optional operation).
   * @param index The index.
   * @param element The element.
   */
  public void add(int index, Object element) {
    if(Compatibility.isJava1_1()) {
      ((ArrayList_1_1)arrayList).add(index, element);
    } else {
      ((java.util.ArrayList)arrayList).add(index, element);
    }
  }

  /**
   * Appends all of the elements in the specified collection to the end of this
   * list, in the order that they are returned by the specified collection's
   * iterator (optional operation).
   * @param c The collection.
   * @return True if the list changed.
   */
  public boolean addAll(Collection c) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).addAll(c);
    }
    Object[] objects = c.toArray();
    ((java.util.ArrayList)arrayList).ensureCapacity(objects.length);
    boolean isChanged = false;
    for(int i=0; i<objects.length; i++) {
      isChanged |= add(objects[i]);
    }
    return isChanged;
  }

  /**
   * Removes the element at the specified position in this list (optional
   * operation).
   * @param index The index of the element to remove.
   * @return The element that was removed.
   */
  public Object remove(int index) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).remove(index);
    }
    return ((java.util.ArrayList)arrayList).remove(index);
  }

  /**
   * Removes the first occurrence in this list of the specified element
   * (optional operation). 
   * @param o The element to remove.
   * @return True if the element was present.
   */
  public boolean remove(Object o) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).remove(o);
    }
    return ((java.util.ArrayList)arrayList).remove(o);
  }

  /**
   * Removes all this collection's elements that are also contained in the
   * specified collection (optional operation).
   * @param c The collection of elements to remove.
   * @return True if the collection was affected by this operation.
   */
  public boolean removeAll(Collection c) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).removeAll(c);
    }
    Object[] objects = c.toArray();
    boolean isChanged = false;
    for(int i=0; i<objects.length; i++) {
      isChanged |= remove(objects[i]);
    }
    return isChanged;
  }

  /**
   * Replaces the element at the specified position in this list with the
   * specified element (optional operation).
   * @param index The index at which to set the element.
   * @param element The element to set.
   * @return The element previously located at the specified position.
   */
  public Object set(int index, Object element) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).set(index, element);
    }
    return ((java.util.ArrayList)arrayList).set(index, element);
  }

  /**
   * Returns the index in this list of the first occurrence of the specified
   * element, or -1 if this list does not contain this element.
   * @param o The element to look for.
   * @return The index.
   */
  public int indexOf(Object o) {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).indexOf(o);
    }
    return ((java.util.ArrayList)arrayList).indexOf(o);
  }

  /**
   * Returns true if this list contains no elements.
   * @return True if this list is empty.
   */
  public boolean isEmpty() {
    if(Compatibility.isJava1_1()) {
      return ((ArrayList_1_1)arrayList).isEmpty();
    }
    return ((java.util.ArrayList)arrayList).isEmpty();
  }

  /**
   * Increases the capacity of this ArrayList instance, if necessary, to ensure
   * that it can hold at least the number of elements specified by the minimum
   * capacity argument.
   * @param minCapacity The minimum capacity.
   */
  public void ensureCapacity(int minCapacity) {
    if(Compatibility.isJava1_1()) {
      ((ArrayList_1_1)arrayList).ensureCapacity(minCapacity);
    }
    ((java.util.ArrayList)arrayList).ensureCapacity(minCapacity);
  }


}
